using FlexibleGameServer;
using FlexibleGameServer.ObjectsNetworkingModel;
using FlexibleGameServer.Unity3d.Networking;
using FlexibleGameServer.Vectors;
using UnityEngine;

public class ComponentTransform : FGSClientObservedComponent
{
    private FloatVector3 _position;

    [ReadField(NetSerializeGroups.TcpBroadcast)]
    [WriteField(NetSerializeGroups.Tcp)]
    FloatVector3 Position
    {
        get
        {
            return _position;
        }
        set
        {
            if (!Point.IsOwner) _position = value;
        }
    }

    private FloatVector3 _rotation;

    [ReadField(NetSerializeGroups.TcpBroadcast)]
    [WriteField(NetSerializeGroups.Tcp)]
    FloatVector3 Rotation
    {
        get
        {
            return _rotation;
        }
        set
        {
            if (!Point.IsOwner) _rotation = value;
        }
    }

    protected void FixedUpdate()
    {
        if ((Point != null) && !Point.IsOwner)
        {
            transform.position = Vector3.Lerp(transform.position, _position.ToUnityVector3(), 0.05f);
            transform.rotation = Quaternion.Lerp(transform.rotation, Quaternion.Euler(_rotation.ToUnityVector3()), 0.05f);
        }
        else
        {
            transform.position += -Vector3.left * Input.GetAxis("Horizontal") * 0.01f;
            transform.position += Vector3.forward * Input.GetAxis("Vertical") * 0.01f;
            _position = transform.position.ToFloatVector3();
            _rotation = transform.rotation.eulerAngles.ToFloatVector3();
        }
    }

    public override void OnNetInitializing(DataBuffer.BinaryReader reader)
    {
        base.OnNetInitializing(reader);
        if (!Point.IsOwner)
        {
            _position = reader.ReadFloatVector3();
            _rotation = reader.ReadFloatVector3();
            transform.position = _position.ToUnityVector3();
            transform.rotation = Quaternion.Euler(_rotation.ToUnityVector3());
        }
    }


}
